import React, { useState, useEffect, ReactNode } from 'react';
import { StatValue } from '../components/Stats/StatCard';
import { displayRecentPerformanceSamples, getVoteAccounts, getTransactionCount, SAMPLE_HISTORY_HOURS, MAINNET_ENDPOINT, INTERNAL_MAINNET_ENDPOINT } from './hooks/useTransactionStats';
import Loader from '../molecules/SVGs/Loader';

const isDevelopment: boolean = typeof window !== 'undefined' && window.location.hostname === 'localhost';

type FetchDataFunction = () => Promise<React.ReactElement<any, string | React.JSXElementConstructor<any>> | any>;

interface Stat {
  stat?: string;
  value?: StatValue
}

const statsValueRenderer = (stat: Stat) => {
  const rpcNodeURL = isDevelopment ? INTERNAL_MAINNET_ENDPOINT : MAINNET_ENDPOINT;
  const abortController = new AbortController();
  const abortSignal = abortController.signal;

  const [displayValue, setDisplayValue] = useState<string | ReactNode | undefined>(undefined);

  useEffect(() => {
    const abortController = new AbortController();
    const abortSignal = abortController.signal;

    const endpointFunctions: Record<string, Record<string, FetchDataFunction>> = {
      'Explorer API': {
        'Transactions per second': () => displayRecentPerformanceSamples(abortSignal, SAMPLE_HISTORY_HOURS, rpcNodeURL),
        'Total Transactions': () => getTransactionCount(abortSignal, rpcNodeURL),
        'Validator Nodes': () => getVoteAccounts(abortSignal, rpcNodeURL),
        // Add other endpoints and their functions here
      }
    };

    const fetchStatValue = async () => {
      if (!stat || !stat.value) {
        setDisplayValue(stat.stat);
        return;
      }

      const { statType, staticValue, dynamicValueSource, dynamicValueEndpoint } = stat.value;

      if (statType === 'static') {
        setDisplayValue(staticValue);
        return;
      }

      if (statType !== 'dynamic' || !dynamicValueSource || !dynamicValueEndpoint) {
        console.error('Unsupported stat type or missing dynamic endpoint');
        return;
      }

      try {
        const fetchDataFunction = endpointFunctions[dynamicValueSource][dynamicValueEndpoint];
        if (!fetchDataFunction) {
          console.error('Invalid API endpoint:', dynamicValueEndpoint);
          return;
        }

        const data = await fetchDataFunction();
        setDisplayValue(data);
      } catch (error) {
        console.error('Error fetching data from API:', error);
      } finally {
        abortController.abort(); // Clean up
      }
    };

    fetchStatValue();

    return () => abortController.abort();
  }, [stat.value]);



  return (
    <>
      {displayValue ? <>{displayValue}</> : <Loader />}
    </>
  );
};

export default statsValueRenderer;
